#
# Project Atomic Unit Test Example Template
#

import unittest


class TestAtomicUnit(unittest.TestCase):
    def test_unit_desired_behavior(self):
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()
